@@ -3,7 +3,7 @@
 # Project: AutoArchive
 # License: GNU GPLv3
 #
-# Copyright (C) 2003 - 2021 Róbert Čerňanský
+# Copyright (C) 2003 - 2022 Róbert Čerňanský
 
 
 
@@ -16,7 +16,7 @@ __all__ = ["ArchiverManipulator"]
 
 
 # {{{ INCLUDES
-
+import glob
 import os
 from datetime import date
 
@@ -102,6 +102,8 @@ class ArchiverManipulator:
         :raise RuntimeError: If the backup operation was aborted due to a fatal error.
         :raise OSError: If a system error occurred while making the backup."""
 
+        self.__validatePathThatShallBeBackedUp()
+
         backupDefinition = self.__createAndPopulateBackupDefinition()
 
         self.__backupOperationStatus = True
@@ -286,21 +288,55 @@ class ArchiverManipulator:
 
 
 
+    def __validatePathThatShallBeBackedUp(self):
+        path = self.__archiveSpec[ArchiveSpecOptions.PATH]
+        if not os.path.exists(path):
+            raise OSError(
+                f"Directory \"{path}\" configured in the \"{ArchiveSpecOptions.PATH}\" option does not exists.")
+        if not os.path.isdir(path):
+            raise OSError(f"Path \"{path}\" configured in the \"{ArchiveSpecOptions.PATH}\" option is not a directory.")
+        if not os.access(path, os.R_OK | os.X_OK, effective_ids = True):
+            raise OSError(
+                f"Directory \"{path}\" configured in the \"{ArchiveSpecOptions.PATH}\" option is not accessible.")
+
+
+
     def __createAndPopulateBackupDefinition(self):
         backupDefinition = BackupDefinition()
         backupDefinition.backupId = self.__archiveSpec[ArchiveSpecOptions.NAME]
         backupDefinition.backupType = self.__ARCHIVER_TYPE_TO_BACKUP_TYPE_MAP[self.__archiveSpec[Options.ARCHIVER]]
         backupDefinition.destination = self.__archiveSpec[Options.DEST_DIR]
         backupDefinition.root = self.__archiveSpec[ArchiveSpecOptions.PATH]
-        backupDefinition.includeFiles = self.__archiveSpec[ArchiveSpecOptions.INCLUDE_FILES]
-        backupDefinition.excludeFiles = self.__archiveSpec[ArchiveSpecOptions.EXCLUDE_FILES]
+        backupDefinition.includeFiles = self.__expandWilds(self.__archiveSpec[ArchiveSpecOptions.INCLUDE_FILES])
+        backupDefinition.excludeFiles = self.__expandWilds(self.__archiveSpec[ArchiveSpecOptions.EXCLUDE_FILES])
         return backupDefinition
 
 
 
+    def __expandWilds(self, files):
+        "Expand wildcards in list of files."
+
+        cwdSave = os.getcwd()
+        os.chdir(os.path.expanduser(self.__archiveSpec[ArchiveSpecOptions.PATH]))
+
+        expandedFiles = set()
+        for fileName in files:
+            expanded = glob.glob(fileName)
+            if expanded:
+                expandedFiles.update(expanded)
+            else:
+                if self.__componentUi:
+                    self.__componentUi.showWarning(
+                        str.format("Cannot access \"{}\". No such file or directory.", fileName))
+
+        os.chdir(cwdSave)
+        return frozenset(expandedFiles)
+
+
+
     def __reportUnsupportedOptions(self):
         for option in self.__OPTION_TO_FEATURE_MAP:
-            if self.__archiveSpec.getRawValue(option) is not None and not self.isOptionSupported(option):
+            if self.__archiveSpec.isOptionPresent(option) and not self.isOptionSupported(option):
                 self.__componentUi.showWarning(str.format(
                     "Option \"{}\" is not supported by the archiver of type \"{}\".",
                     option, OptionsUtils.archiverTypeToStr(self.__archiveSpec[Options.ARCHIVER])))
