@@ -3,7 +3,7 @@
 # Project: AutoArchive
 # License: GNU GPLv3
 #
-# Copyright (C) 2003 - 2016 Róbert Čerňanský
+# Copyright (C) 2003 - 2022 Róbert Čerňanský
 
 
 
@@ -17,7 +17,6 @@ __all__ = ["ArchivingApplication"]
 
 # {{{ INCLUDES
 
-import itertools
 from datetime import date
 
 from AutoArchive._infrastructure.utils import Utils
@@ -89,8 +88,11 @@ class ArchivingApplication:
         :param selectedArchiveSpecs: :term:`archive specification files <archive specification file>` for which backups
            shall be created.
         :type selectedArchiveSpecs: ``Sequence<ArchiveSpecInfo>``"""
-        
-        selectedArchiveSpecs = self.__appendAllArchiveSpecsIfSelected(selectedArchiveSpecs)
+
+        archiveSpecsForProcessing = self.__getArchiveSpecsForProcessing(selectedArchiveSpecs)
+        if archiveSpecsForProcessing is None:
+            return False
+
         self.__actionResult = self.__ActionResults.Successful
         self.__componentUi.messageShown += self.__onMessageShown
 
@@ -98,9 +100,9 @@ class ArchivingApplication:
             self.__commandExecutor.executeBeforeAllCommand()
         except OSError as ex:
             self.__componentUi.showError(ex.args[0])
-            return
+            return False
 
-        for specName, specFile in selectedArchiveSpecs:
+        for specName, specFile in archiveSpecsForProcessing:
             processingArchSpec = specName or specFile
             self.__componentUi.setProcessingArchSpec(processingArchSpec)
 
@@ -114,7 +116,7 @@ class ArchivingApplication:
             self.__commandExecutor.executeAfterAllCommand()
         except OSError as ex:
             self.__componentUi.showError(ex.args[0])
-            return
+            return False
 
         self.__componentUi.setProcessingArchSpec(None)
         self.__componentUi.messageShown -= self.__onMessageShown
@@ -157,13 +159,16 @@ class ArchivingApplication:
            listed.
         :type selectedArchiveSpecs: ``Sequence<ArchiveSpecInfo>``"""
 
-        archiveSpecs = self.__appendAllArchiveSpecsIfSelected(selectedArchiveSpecs)
+        archiveSpecsForProcessing = self.__getArchiveSpecsForProcessing(selectedArchiveSpecs)
+        if archiveSpecsForProcessing is None:
+            return False
+
         self.__actionResult = self.__ActionResults.Successful
         self.__componentUi.messageShown += self.__onMessageShown
 
-        orphanedArchives = frozenset(self.__getOrphanedArchives(archiveSpecs))
+        orphanedArchives = frozenset(self.__getOrphanedArchives(archiveSpecsForProcessing))
 
-        for archiveSpecInfo in archiveSpecs:
+        for archiveSpecInfo in archiveSpecsForProcessing:
             processingArchSpec = archiveSpecInfo.name or archiveSpecInfo.path
             self.__componentUi.setProcessingArchSpec(processingArchSpec)
 
@@ -183,7 +188,7 @@ class ArchivingApplication:
                 self.__showStandardArchiveInfo(archiveInfo)
         self.__componentUi.setProcessingArchSpec(None)
 
-        archiveSpecsNames = {spec.name for spec in archiveSpecs}
+        archiveSpecsNames = {spec.name for spec in archiveSpecsForProcessing}
         for orphanedArchiveName in orphanedArchives:
             self.__componentUi.setProcessingArchSpec(orphanedArchiveName)
 
@@ -338,9 +343,12 @@ class ArchivingApplication:
 
         result = True
 
-        archiveSpecs = self.__appendAllArchiveSpecsIfSelected(selectedArchiveSpecs)
-        orphanedArchives = frozenset(self.__getOrphanedArchives(archiveSpecs))
-        knownValidArchiveNames = frozenset(self.__getKnownValidArchiveNames(selectedArchiveSpecs))
+        archiveSpecsForProcessing = self.__getArchiveSpecsForProcessing(selectedArchiveSpecs)
+        if archiveSpecsForProcessing is None:
+            return False
+
+        orphanedArchives = frozenset(self.__getOrphanedArchives(archiveSpecsForProcessing))
+        knownValidArchiveNames = frozenset(self.__getValidArchiveNames(archiveSpecsForProcessing))
         try:
             # names of archive specs which were selected by a name, not by path to .aa file
             archiveSpecNamesSelectedByName = [asi.name for asi in selectedArchiveSpecs if asi.name is not None]
@@ -393,47 +401,43 @@ class ArchivingApplication:
 
 
 
-    def __getOrphanedArchives(self, selectedArchiveSpecs):
-        knownValidArchiveNames = frozenset(self.__getKnownValidArchiveNames(selectedArchiveSpecs))
+    def __getOrphanedArchives(self, archiveSpecs):
         return (name
                 for name in self.__archiving.getStoredArchiveNames()
-                if name not in knownValidArchiveNames)
+                if name not in frozenset(self.__getValidArchiveNames(archiveSpecs)))
 
 
 
     @Utils.uniq
-    def __getKnownValidArchiveNames(self, selectedArchiveSpecs):
-        selectedArchiveSpecFiles = (archSpecInf.path
-                                    for archSpecInf in selectedArchiveSpecs)
-        configuredArchiveSpecsFiles = (archSpecInf.path
-                                       for archSpecInf in self.__archiving.getArchiveSpecs())
-        return self.__archiving.filterValidSpecFiles(
-            itertools.chain(selectedArchiveSpecFiles, configuredArchiveSpecsFiles))
+    def __getValidArchiveNames(self, archiveSpecs):
+        return self.__archiving.filterValidSpecFiles((archSpecInf.path for archSpecInf in archiveSpecs))
 
 
 
-    def __appendAllArchiveSpecsIfSelected(self, selectedArchiveSpecs):
+    def __getArchiveSpecsForProcessing(self, selectedArchiveSpecs):
 
         @Utils.uniq
         def getAllUniqueArchiveSpecs():
             for archiveSpecInfo in selectedArchiveSpecs:
                 yield archiveSpecInfo
 
-            try:
-                empty = True
-                for archiveSpecInfo in self.__archiving.getArchiveSpecs():
-                    empty = False
-                    yield archiveSpecInfo
-                if empty:
-                    self.__componentUi.showWarning("No configured archive specification files were found.")
-            except RuntimeError as ex:
-                self.__componentUi.showError(str.format(
-                            "An error occurred while obtaining the list of all archive specification files: {}", ex))
+            empty = True
+            for archiveSpecInfo in self.__archiving.getArchiveSpecs():
+                empty = False
+                yield archiveSpecInfo
+            if empty:
+                self.__componentUi.showWarning("No configured archive specification files were found.")
 
 
 
         if len(selectedArchiveSpecs) == 0 or self.__configuration[Options.ALL]:
-            return tuple(getAllUniqueArchiveSpecs())
+            try:
+                allUniqueArchiveSpecs = tuple(getAllUniqueArchiveSpecs())
+            except (RuntimeError, OSError) as ex:
+                self.__componentUi.showError(str.format(
+                    "An error occurred while obtaining the list of all archive specification files: {}", ex))
+                return None
+            return allUniqueArchiveSpecs
         else:
             return selectedArchiveSpecs
 
